# Import Data
SexAtt <- read.table("http://eur.academia.edu/documents/0009/0840/sexualat.dat")
#SexAtt <- read.table("C:/Users/Dimitris/Documents/PackagesSource/ltm/Data/sexualat.dat")
names(SexAtt) <- c("divorce", "sexdisc", "premar", "exmar", "gaysex", 
"gayscho", "gayhied", "gaypubl", "gayfadop", "gaymadop")

head(SexAtt)

# load 'ltm' package
library(ltm)

                        ############################
                        # IRT for Dichotomous Data #
                        ############################


##########################
# Descriptive Statistics #
##########################

dsc <- descript(SexAtt)
dsc
plot(dsc, type = "b", lty = 1, pch = 1:10, col = 1:10, lwd = 2, cex = 1.1, xlim = c(-0.5, 9))
legend("left", names(SexAtt), pch = 1:10, col = 1:10, lty = 1, lwd = 2, cex = 1.1, bty = "n")


####################
# Fit Rasch Models #
####################

# fix discrimination to 1
fit.rasch1 <- rasch(SexAtt, constraint = cbind(ncol(SexAtt) + 1, 1))
summary(fit.rasch1)
# items ordered by difficulty, and
# probability of positive response by the average
# individual
coef(fit.rasch1, prob = TRUE, order = TRUE)

# unconstrained Rasch model
fit.rasch2 <- rasch(SexAtt)
summary(fit.rasch2)


# perform a Likelihood Ratio Test between the two models
anova(fit.rasch1, fit.rasch2)


# Bootstrapped Pearson chi-squared goodness-of-fit tests
GoF.rasch(fit.rasch1, B = 100)
GoF.rasch(fit.rasch2, B = 100)

# check the fit to the three-way margins
margins(fit.rasch2, type = "two-way")
margins(fit.rasch2, type = "three-way")

# Item Characteristic Curves for the unconstrained Rasch model
plot(fit.rasch2, legend = TRUE, pch = rep(1:2, each = 5), xlab = "Attitude",
    col = rep(1:5, 2), lwd = 2, cex = 1.2, sub = paste("Call: ", deparse(fit.rasch2$call)))

# Item Characteristic Curves for specific items
plot(fit.rasch2, items = c(1, 3, 5, 7), xlab = "Attitude", lwd = 2, cex = 1.2, legend = TRUE, 
    sub = paste("Call: ", deparse(fit.rasch2$call)))

# Item Information Curves for the two unconstrained Rasch model
plot(fit.rasch2, type = "IIC", legend = TRUE, pch = rep(1:2, each = 5), xlab = "Attitude",
     col = rep(1:5, 2), lwd = 2, cex = 1.2, sub = paste("Call: ", deparse(fit.rasch2$call)))

# Test Information Functions for the two models
info1 <- plot(fit.rasch1, type = "IIC", items = 0, plot = FALSE)
info2 <- plot(fit.rasch2, type = "IIC", items = 0, plot = FALSE)
# put them in the same plot
plot(range(info1[, "z"]), range(info1[, "info"], info2[, "info"]), type = "n",
     xlab = "Attitude", ylab = "Information", main = "Test Information Functions")
lines(info1, lwd = 2)
lines(info2, lwd = 2, lty = 2)
legend("topleft", c("Rasch model with discrimination equal to 1", "Rasch model"), lty = 1:2, lwd = 2, bty = "n")


########################################
# Fit the Two-Parameter Logistic Model #
########################################

fit.2pl <- ltm(SexAtt ~ z1)
summary(fit.2pl)

# perform a LRT with the unconstrained Rasch model
anova(fit.rasch2, fit.2pl)

# check the fit to the two-way margins
margins(fit.2pl)
# check the fit to the three-way margins
margins(fit.2pl, type = "three-way")


#################################
# Fit the Three-Parameter Model #
#################################

tpm(SexAtt, type = "rasch", constraint = cbind(1, 3, 1))
tpm(SexAtt, type = "rasch")
tpm(SexAtt)


##################################################
# Fit Latent Trait Model with 2 latent variables #
##################################################

# use 21 quadrature points instead of the default 15
fit.lt2l <- ltm(SexAtt ~ z1 + z2, control = list(GHk = 21))
summary(fit.lt2l)

# Standardized Loadings scatterplot
plot(fit.lt2l, type = "loadings")

# Item Characteristic Surfaces
plot(fit.lt2l, ticktype = "detailed", theta = 30, phi = 30, expand = 0.5, d = 2, 
     cex = 0.7, col = "lightblue")

# include an interaction term between the 2 latent variables;
# use 21 quadrature points instead of the default 15
fit.ltnl <- ltm(SexAtt ~ z1 * z2, control = list(GHk = 21))
summary(fit.ltnl)

# perform a LRT with the linear 2-factor model
anova(fit.lt2l, fit.ltnl)


##############################################################################################
##############################################################################################

                        ###########################
                        # IRT for Polytomous Data #
                        ###########################


##########################
# Descriptive Statistics #
##########################

descript(Environment)

rcor.test(Environment, method = "kendall")

##########################################
# Rasch Generalized Partial Credit Model #
##########################################

# the Rasch version of the Generalized Partial Credit Model 
# assumes a common discrimination parameter across items fixed at 1
fit.gpcm1 <- gpcm(Environment, constraint = "rasch")
fit.gpcm1
summary(fit.gpcm1)
# you can ask for sandwich standard errors using
# the 'robust.se' argument
summary(fit.gpcm1, robust.se = TRUE)


# check the fit on the margins
margins(fit.gpcm1)
margins(fit.gpcm1, type = "three-way")

# the fit can also be assessed using the
# Pearson chi-squared goodness-of-fit measure;
# this can be done using the GoF.gpcm() function.
# By default the function produces an estimate to the
# p-value using a parametetric Boostrap approach.
# This is safer than using the asymptotic chi-squared
# distribution but it requires more computing time.
# In the first call below we use 'simulate.p.value = FALSE'
# the specifies to use the asymptotic distribution.
# In the second call the Bootstrap approach is used.
GoF.gpcm(fit.gpcm1, simulate.p.value = FALSE)
GoF.gpcm(fit.gpcm1, B = 199) # this will take some time depending on your machine


########################################
# 1PL Generalized Partial Credit Model #
########################################

# the 1PL version of the Generalized Partial Credit Model 
# assumes a common discrimination parameter across items
fit.gpcm2 <- gpcm(Environment, constraint = "1PL")
summary(fit.gpcm2)


####################################
# Generalized Partial Credit Model #
####################################

# the unconstrained version of the Generalized Partial Credit 
# assumes a different discrimination parameter per item
fit.gpcm3 <- gpcm(Environment)


# Likelihood Ratio Test between constrained and unconstrained models;
# here we test if the discrimination parameter should be fixed at 1
anova(fit.gpcm1, fit.gpcm2)
# here we test if the discrimination parameter is equal among the 6 items
anova(fit.gpcm2, fit.gpcm3)


#####################################
# constrained Graded Response Model #
#####################################

# the constrained Graded Response Model assumes a common
# discrimination parameter across items;
# 'Hessian = TRUE' is used to obtain standard errrors
fit.grm1 <- grm(Environment, constrained = TRUE, Hessian = TRUE)
fit.grm1
summary(fit.grm1)


#######################################
# unconstrained Graded Response Model #
#######################################

# the unconstrained Graded Response Model assumes a different
# discrimination parameter per item;
# 'Hessian = TRUE' is used to obtain standard errrors
fit.grm2 <- grm(Environment, Hessian = TRUE)
fit.grm2
summary(fit.grm2)

# Likelihood Ratio Test between constrained 
# and unconstrained models
anova(fit.grm1, fit.grm2)


##############################################
# Item Charecteristic and Information Curves #
##############################################

# by default the Item Response Category Characterstic
# Curves are produced; we include a legend
par(mfrow = c(2, 2))
plot(fit.gpcm3, lwd = 2, cex = 1.2, legend = TRUE, cx = "left",
     xlab = "Latent Trait", cex.main = 1.5, cex.lab = 1.3, cex.axis = 1.1)

# to see more clearly the difference in the amount of infromation provided by
# the three models use the following code. In addition, we have also
# included the information from the Graded Response Model for a comparison:
vals1 <- plot(fit.gpcm1, type = "IIC", items = 0, plot = FALSE)
vals2 <- plot(fit.gpcm2, type = "IIC", items = 0, plot = FALSE)
vals3 <- plot(fit.gpcm3, type = "IIC", items = 0, plot = FALSE)
vals4 <- plot(fit.grm1, type = "IIC", items = 0, plot = FALSE)
vals5 <- plot(fit.grm2, type = "IIC", items = 0, plot = FALSE)

values <- cbind(vals1[, "test.info"], vals2[, "test.info"], vals3[, "test.info"], vals4[, "test.info"], vals5[, "test.info"])
matplot(vals1[, "z"], values, type = "l", col = c("black", "red", "blue", "green", "goldenrod1"), 
    xlab = "Latent Trait", ylab = "Information", main = "A Comparison between Different Polytomous IRT Models", 
    lty = 1, lwd = 2, cex.main = 1.5, cex.lab = 1.3, cex.axis = 1.1)
lgd <- c("GPCM (discrimination = 1)", "GPCM (discrimination equal among items)", 
    "GPCM (discrimination different per item)", "GRM (discrimination equal among items)", "GRM (discrimination different per item)")
legend("topleft", lgd, col = c("black", "red", "blue", "green", "goldenrod1"), cex = 1.2, lty = 1, lwd = 2, bty = "n")


# using the function information() you can numerically compare
# the infromation provided by the different IRT models for 
# polytomous data

# discrimination = 1
information(fit.gpcm1, c(-4, 4))

# discrimination equal among items
information(fit.gpcm2, c(-4, 4))
information(fit.grm1, c(-4, 4))

# discrimination different per item
information(fit.gpcm3, c(-4, 4))
information(fit.grm2, c(-4, 4))


#############################
# Latent Variable Estimates #
#############################

# the latent abilities can be estimated using
# the factor.scores() function:

# default is maximum a posteriori
fs <- factor.scores(fit.gpcm3)
fs
# you can also plot the kernel density estimator for
# the latent abilities using the plot method, e.g.,
plot(fs)

# for expected a posteriori use
factor.scores(fit.gpcm3, method = "EAP")

# factor scores can also be computed for specific response patterns;
# to do that use the 'resp.patterns' argument, e.g.,
factor.scores(fit.gpcm3, resp.patterns = rbind("A" = c(1,2,3,NA,2,1), "B" = c(1,1,1,2,2,2)))
